/* Copyright (C) 2001  Martin J. Muench <muench@gmc-online.de> */

#include "config.h"

extern void sendpacket(char *, char *);
extern void start_curses();
extern void end_curses();
extern void enigma(char *, int);

int main (int argc, char **argv) {

 int numActiveFDs, maxFD, null=0, auth=0;
 char host[256], *payload, *stext, *nick, text[CHARPERLINE];
 u_char buf[1024];

 struct hostent	*hostent;
 FILE *fp=stdin;
 fd_set readset;


if (argc < 3) {
 
 /* Ok, give a little help */
  if(argv[1] && !strcmp(argv[1], "-h"))

/******** Help ***********/
	fprintf(stderr, "
Usage: %s <host> <icmp_code> [nick]\n
<host>\t\tHostname to connect to
<icmp_code>\tICMP-Code to use
<nick>\t\t(optional) Nick of Chatpartner\n
Available ICMP-Codes:\n
 [0] Echo Reply\t\t\t [8] Echo Request
 [5] Redirect\t
 [9] Router advertisement\t[10] Router solicitation
[13] Timestamp request\t\t[14] Timestamp reply
[15] Information request\t[16] Information reply
[17] AddressMask request\n[18] AdressMask reply
\n", argv[0]);
/*************************/

  else 
/******** Usage ***********/
	fprintf(stderr, "
icmpchat %s
\nUsage: %s <host> <icmp_code> [nick of chatpartner]
       %s -h  for help
\n", VERSION, argv[0], argv[0]);\
/**************************/
exit(1);
}
 /* ICMP_Code */
 icmpcode=atoi(argv[2]); 

 /* Nick=Argument2 if given, else Nick=Hostname */
 if(!argv[3]) nick=strdup(argv[1]); 
  else nick=strdup(argv[3]); 

 /* Start the fancy Interface */
 start_curses();
 
 /* Lookup hostname */
 if (!(hostent = gethostbyname(argv[1]))) {
	end_curses(); perror("Hostname:"); exit(-1);
 }
 strncpy(host,inet_ntoa(*(struct in_addr*)hostent->h_addr_list[0]),256);

 /* Print out welcome screen */
 wprintw(output, "\nICMP-Chat %s\n-------------------------------------------------\n", VERSION);
 wprintw(output, "
::::> *******************
::::> Available Commands:
::::> -------
::::> /about\t- About
::::> /help\t- Help
::::> /quit\t- Quit
::::> *******************\n
::::>  connecting to %s...", host);
 wrefresh(output);
 
 /* Open Raw socket and set socket options */
 if ((rawsock=socket(AF_INET,SOCK_RAW,IPPROTO_ICMP)) < 0) { 
 	end_curses(); perror("RawSocket:"); exit(-1); 
 }
 if (setsockopt(rawsock,IPPROTO_IP,IP_HDRINCL,&null,sizeof(null)) < 0) { 
 	close(rawsock); end_curses(); perror("SetSockOpt:"); exit(-1); 
 }

 /* Return to user privileges */
 setuid(getuid());
 
 maxFD = rawsock;
 FD_ZERO(&readset);

 /* Send first authentication packet */
 sendpacket(PING, host);

 for(;;) {
 
 	/* Set FDs */
	FD_SET(rawsock, &readset);
	FD_SET(0, &readset);

	/* Select FDs */
	if ((numActiveFDs = select((maxFD + 1), &readset, NULL, NULL, NULL)) < 0) {
		end_curses(); perror("Select:"); exit(-1);
	}
	
	/* Data on rawsock ? */
	if (FD_ISSET(rawsock, &readset)) {

	  /* Receive the data */
	  if (recvfrom(rawsock,buf,1024,0,(struct sockaddr *)&saddr,&null) < 0) break;

		/* Set structure to detect the payload */
		ip=(struct ip *)buf;
		icmp=(struct icmp *)&buf[ip->ip_hl<<2];
		
		  /* Is the source of the package the defined? */
		  if (!strcmp(host,inet_ntoa(ip->ip_src))) {
		  
		  	/* Set the payload */
			payload = (buf + sizeof(struct ip) + sizeof(struct icmp));

			/* Decrypt Payload */
			enigma(payload, strlen(payload));

			/* If SessionID is correct, analyze the data */
			if(!strncmp(enicrypt, SID, strlen(SID))) {
			
				/* Strip text */
				stext=(enicrypt + strlen(SID));
				
				
				/* Does payload contain authentication string 1? */
				if(!strcmp(stext,PING) && !auth) {
	
					/* Send authentication string 1 */
					sendpacket(PONG,host); auth=1; 			  
	
					/* Got connected */
					wprintw(output, "\n::::>  %s is ready to chat\n\n", nick); 
					wrefresh(output);
				}
			
				/* Does payload contain authentication string 2? */
				else if(!strcmp(stext,PONG) && (!auth || auth==1)) {
	
					/* First authentication message? */
					if (!auth) {
						wprintw(output, "\n::::>  %s is ready to chat\n\n", nick);
			                	wrefresh(output); auth=2;
			 			sendpacket(PONG,host);
					}
				}
		
				/* Chartpartner disconnected? */
				else if(!strcmp(stext,"/quit")) {
					wprintw(output, "\n::::> %s has left the chat\n\n", nick);
					wrefresh(output);
				}

				/* Authentication done, chat */
				else if (auth) {	
					wprintw(output, "%s: %s\n", nick, stext);
					wrefresh(output);
				}
		  }
		}
	}
	
	/* Data on stdin? */
	else if (FD_ISSET(fileno(fp), &readset)) { 
	  if (wgetnstr(input, text, CHARPERLINE - 1)) exit;
	  else {

		/* The professional ;-) quit-command */
	  	if(!strcmp(text,"/quit")) { 
	  		sendpacket("/quit",host);	
	  		end_curses(); close(rawsock); exit(1); 
	  	}
		/* Need Help? */
		else if(!strcmp(text,"/help")) {
			wprintw(output, "
::::> *******************
::::> Available Commands:
::::> -------
::::> /about\t- About
::::> /help\t- Help
::::> /quit\t- Quit
::::> *******************\n");
			wrefresh(output);
		}   
		else if(!strcmp(text,"/about")) {
			wprintw(output, "
::::> ICMP-Chat %s
::::> Get it at: http://mjm.gmc-online.de\n", VERSION);
			wrefresh(output);
		}
		else {
			/* Print text */
			wprintw(output, "You: %s\n", text);
                	wrefresh(output);

                	/* Send text */
			sendpacket(text,host);
		
		}
		
		/* Clear Input */
		werase(input);  
		wmove(input, 0, 0);
		wrefresh(input);
	  }
 	}
 }
exit(0);
}
